@@ -6,7 +6,7 @@
  * See http://opensource.adnovum.ch/mod_qos/ for further
  * details.
  *
- * Copyright (C) 2007-2012 Pascal Buchbinder
+ * Copyright (C) 2007-2013 Pascal Buchbinder
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,11 +25,12 @@
  *
  */
 
-static const char revision[] = "$Id: qsgeo.c,v 1.8 2012/12/18 19:32:50 pbuchbinder Exp $";
+static const char revision[] = "$Id: qsgeo.c,v 1.12 2013/11/27 07:09:06 pbuchbinder Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <error.h>
 #include <string.h>
 #include <time.h>
 #include <sys/time.h>
@@ -56,6 +57,22 @@ static const char revision[] = "$Id: qsgeo.c,v 1.8 2012/12/18 19:32:50 pbuchbind
 #define QS_GEO_PATTERN_D "\"([0-9]+)\",\"([0-9]+)\",\"([A-Z0-9]{2})\",\"(.*)\""
 // 182.12.34.23
 #define IPPATTERN "([0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3})[\"'\x0d\x0a, ]+"
+#define IPPATTERN2 "([0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3})[\"'\x0d\x0a,; ]+"
+
+static int m_inject = 0;
+
+typedef struct {
+  unsigned long start;
+  char *c;
+} qos_inj_t;
+
+static const qos_inj_t m_inj[] = {
+  { 167772160, "\"10.0.0.0\",\"10.255.255.255.255\",\"167772160\",\"184549375\",\"PV\",\"private network\"" },
+  { 2130706432, "\"127.0.0.0\",\"127.255.255.255\",\"2130706432\",\"2147483647\",\"LO\",\"local loopback\"" },
+  { 2886729728, "\"172.16.0.0\",\"172.31.255.255\",\"2886729728\",\"2887778303\",\"PV\",\"private network\"" },
+  { 3232235520, "\"192.168.0.0\",\"192.168.255.255\",\"3232235520\",\"3232301055\",\"PV\",\"private network\"" },
+  { 0, NULL }
+};
 
 typedef struct {
   unsigned long start;
@@ -155,7 +172,7 @@ static void usage(const char *cmd, int man) {
   if(man) {
     printf(".SH SYNOPSIS\n");
   }
-  qs_man_print(man, "%s%s -d <path> [-s] [-ip <ip>]\n",  man ? "" : "Usage: ", cmd);
+  qs_man_print(man, "%s%s -d <path> [-l] [-s] [-ip <ip>]\n",  man ? "" : "Usage: ", cmd);
   printf("\n");
   if(man) {
     printf(".SH DESCRIPTION\n");
@@ -175,13 +192,19 @@ static void usage(const char *cmd, int man) {
   if(man) printf("\n.TP\n");
   qs_man_print(man, "  -d <path>\n");
   if(man) printf("\n");
-  qs_man_print(man, "     Specifies the path to the geographical database files (CSV file\n");
-  qs_man_print(man, "     containing IP address ranges and country codes.\n");
+  qs_man_print(man, "     Specifies the path to the geographical database files (CSV\n");
+  qs_man_print(man, "     file containing IP address ranges and country codes).\n");
   if(man) printf("\n.TP\n");
   qs_man_print(man, "  -s\n");
   if(man) printf("\n");
   qs_man_print(man, "     Writes a summary of the requests per country only.\n");
   if(man) printf("\n.TP\n");
+  qs_man_print(man, "  -l\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     Writes the database to stdout (ignoring stdin) inserting\n");
+  qs_man_print(man, "     local (127.*) and private (10.*, 172.16*, 192.168.*)\n");
+  qs_man_print(man, "     network addresses.\n");
+  if(man) printf("\n.TP\n");
   qs_man_print(man, "  -ip <ip>\n");
   if(man) printf("\n");
   qs_man_print(man, "     Resolves a single IP address instead of processing a log file.\n");
@@ -258,7 +281,9 @@ static qos_geo_t *qos_loadgeo(apr_pool_t *pool, const char *db, int *size, char
   qos_geo_t *last = NULL;
   int lines = 0;
   char line[HUGE_STRING_LEN];
+  char buf[HUGE_STRING_LEN];
   FILE *file = fopen(db, "r");
+  const qos_inj_t *inj = m_inj;
   *size = 0;
   if(!file) {
     return NULL;
@@ -289,6 +314,9 @@ static qos_geo_t *qos_loadgeo(apr_pool_t *pool, const char *db, int *size, char
     lines++;
     if(strlen(line) > 0) {
       int plus = 0;
+      if(m_inject) {
+        strcpy(buf, line);
+      }
       if(regexec(&pregd, line, MAX_REG_MATCH, ma, 0) == 0) {
 	plus = 1;
       }
@@ -299,10 +327,18 @@ static qos_geo_t *qos_loadgeo(apr_pool_t *pool, const char *db, int *size, char
 	g->start = atoll(&line[ma[1].rm_so]);
 	g->end = atoll(&line[ma[2].rm_so]);
 	g->c[0] = '\0';
+        if(m_inject) {
+          if(inj->start && (g->start > inj->start)) {
+            printf("%s\n", inj->c);
+            inj++;
+          } else {
+            printf("%s", buf);
+          }
+        }
 	strncpy(g->country, &line[ma[3].rm_so], 2);
 	if(last) {
 	  if(g->start < last->start) {
-	    *msg = apr_psprintf(pool, "wrong order/lines not storted (line %d)", lines);
+	    *msg = apr_psprintf(pool, "wrong order/lines not sorted (line %d)", lines);
 	  }
 	}
 	if(plus) {
@@ -351,6 +387,8 @@ int main(int argc, const char * const argv[]) {
       }
     } else if(strcmp(*argv, "-s") == 0) {
       stat = 1;
+    } else if(strcmp(*argv, "-l") == 0) {
+      m_inject = 1;
     } else if(strcmp(*argv,"-h") == 0) {
       usage(cmd, 0);
     } else if(strcmp(*argv,"--help") == 0) {
@@ -371,12 +409,18 @@ int main(int argc, const char * const argv[]) {
   }
 
   rc = nice(10);
+  if(rc == -1) {
+    fprintf(stderr, "ERROR, failed to change nice value: %s\n", strerror(errno));
+  }
 
   geo = qos_loadgeo(pool, db, &size, &msg);
   if(geo == NULL || msg != NULL) {
     fprintf(stderr, "failed to load database: %s\n", msg ? msg : "-");
     exit(1);
   }
+  if(m_inject) {
+    exit(0);
+  }
   
   if(ip) {
     qos_geo_t *pB;
@@ -402,13 +446,23 @@ int main(int argc, const char * const argv[]) {
     apr_pool_t *tmp;
     char line[HUGE_STRING_LEN];
     regex_t preg;
+    regex_t preg2;
     regmatch_t ma[MAX_REG_MATCH];
     apr_pool_create(&tmp, NULL);
     if(regcomp(&preg, IPPATTERN, REG_EXTENDED)) {
       exit(1);
     }
+    regcomp(&preg2, IPPATTERN2, REG_EXTENDED);
     while(fgets(line, sizeof(line), stdin) != NULL) {
-      if(regexec(&preg, line, MAX_REG_MATCH, ma, 0) == 0) {
+      int match = regexec(&preg, line, MAX_REG_MATCH, ma, 0);
+      if(match != 0) {
+        char *dx = strchr(line, ';');
+        if(dx && ((dx - line) <= 15)) {
+          // file starts probably with <ip>; => a qslog -pc file?
+          match = regexec(&preg2, line, MAX_REG_MATCH, ma, 0);
+        }
+      }
+      if(match == 0) {
         unsigned long search;
         prev = line[ma[1].rm_eo];
         line[ma[1].rm_eo] = '\0';
@@ -436,15 +490,39 @@ int main(int argc, const char * const argv[]) {
           /* modifies each log line inserting the country code
            */
           char cr = prev;
+          char delw[2];
+          char delx[2];
+          delw[1] = '\0';
+          delw[0] = ' ';
+          delx[1] = '\0';
+          delx[0] = ' ';
+          if(line[ma[1].rm_eo+1] == ' ') {
+            delx[0] = '\0';
+          }
+          if(line[ma[1].rm_eo+1] == ';') {
+            delx[0] = ';';
+          }
           if(prev <= CR) {
             prev = ' ';
           }
+          if(prev == ' ') {
+            delw[0] = '\0';
+          }
+          if(prev == ';') {
+            delw[0] = '\0';
+            delx[0] = ';';
+          }
           if(pB) {
-            printf("%s%c%s%s%s%s", line, prev, prev == ' ' ? "" : " ", pB->country,
-                   line[ma[1].rm_eo+1] == ' ' ? "" : " ", &line[ma[1].rm_eo+1]);
+            printf("%s%c%s%s%s%s", line, prev,
+                   delw,
+                   pB->country,
+                   delx,
+                   &line[ma[1].rm_eo+1]);
           } else {
-            printf("%s%c%s--%s%s", line, prev, prev == ' ' ? "" : " ",
-                   line[ma[1].rm_eo+1] == ' ' ? "" : " ", &line[ma[1].rm_eo+1]);
+            printf("%s%c%s--%s%s", line, prev,
+                   delw,
+                   delx,
+                   &line[ma[1].rm_eo+1]);
           }
           if(cr <= CR) {
             printf("\n");
