@@ -28,9 +28,10 @@
  *
  */
 
-static const char revision[] = "$Id: qslog.c,v 1.61 2013/06/12 19:05:25 pbuchbinder Exp $";
+static const char revision[] = "$Id: qslog.c,v 1.71 2013/11/27 07:09:06 pbuchbinder Exp $";
 
 #include <stdio.h>
+#include <error.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <errno.h>
@@ -72,7 +73,9 @@ static const char revision[] = "$Id: qslog.c,v 1.61 2013/06/12 19:05:25 pbuchbin
 typedef struct {
   long request_count;
   long error_count;
+  long long byte_count;
   long long duration;
+  long long duration_count_ms;
   long duration_0;
   long duration_1;
   long duration_2;
@@ -85,9 +88,20 @@ typedef struct {
   long status_3;
   long status_4;
   long status_5;
+  long status_304;
   long connections;
   apr_table_t *events;
   apr_pool_t *pool;
+  long get;
+  long post;
+  long html;
+  long img;
+  long cssjs;
+  long other;
+  time_t start_s;
+  time_t end_s;
+  long firstLine;
+  long lastLine;
 } client_rec_t;
 
 typedef struct stat_rec_st {
@@ -160,11 +174,13 @@ static int   m_offline_data = 0;
 static char  m_date_str[MAX_LINE];
 static int   m_mem = 0;
 static int   m_avms = 0;
+static int   m_ct = 0;
 static int   m_customcounter = 0;
 static apr_table_t *m_client_entries = NULL;
 static int   m_max_client_entries = 0;
 static int   m_offline_count = 0;
-/* debug */
+static int   m_methods = 0;
+/* debug/offline */
 static long  m_lines = 0;
 static int   m_verbose = 0;
 
@@ -184,6 +200,40 @@ static void qerror(const char *fmt,...) {
   fflush(stderr);
 }
 
+/*
+ * Similar to standard strstr() but we ignore case in this version.
+ * see server/util.c
+ */
+static char *qsstrcasestr(const char *s1, const char *s2) {
+    char *p1, *p2;
+    if (*s2 == '\0') {
+        /* an empty s2 */
+        return((char *)s1);
+    }
+    while(1) {
+        for ( ; (*s1 != '\0') && (tolower(*s1) != tolower(*s2)); s1++);
+        if (*s1 == '\0') {
+            return(NULL);
+        }
+        /* found first character of s2, see if the rest matches */
+        p1 = (char *)s1;
+        p2 = (char *)s2;
+        for (++p1, ++p2; tolower(*p1) == tolower(*p2); ++p1, ++p2) {
+            if (*p1 == '\0') {
+                /* both strings ended together */
+                return((char *)s1);
+            }
+        }
+        if (*p2 == '\0') {
+            /* second string ended, a match */
+            break;
+        }
+        /* didn't find a match here, try starting at next character in s1 */
+        s1++;
+    }
+    return((char *)s1);
+}
+
 /*
  * skip an element to the next space
  */
@@ -739,7 +789,8 @@ static void printAndResetStat(char *timeStr) {
  */
 static void updateClient(apr_pool_t *pool, char *T, char *t, char *D, char *S,
                          char *BI, char *B, char *R, char *I, char *U, char *Q,
-                         char *E, char *k, char *C, long tme) {
+                         char *E, char *k, char *C, char *ct, long tme, long tmems,
+                         char *m) {
   client_rec_t *client_rec;
   const char *id = I; // ip
   if(id == NULL) {
@@ -765,7 +816,9 @@ static void updateClient(apr_pool_t *pool, char *T, char *t, char *D, char *S,
     tid[strlen(id)] = '\0';
     client_rec->request_count = 0;
     client_rec->error_count = 0;
+    client_rec->byte_count = 0;
     client_rec->duration = 0;
+    client_rec->duration_count_ms = 0;
     client_rec->duration_0 = 0;
     client_rec->duration_1 = 0;
     client_rec->duration_2 = 0;
@@ -778,14 +831,28 @@ static void updateClient(apr_pool_t *pool, char *T, char *t, char *D, char *S,
     client_rec->status_3 = 0;
     client_rec->status_4 = 0;
     client_rec->status_5 = 0;
+    client_rec->status_304 = 0;
     client_rec->connections = 0;
     client_rec->events = apr_table_make(pool, 100);
     client_rec->pool = pool;
+    client_rec->get = 0;
+    client_rec->post = 0;
+    client_rec->html = 0;
+    client_rec->img = 0;
+    client_rec->cssjs = 0;
+    client_rec->other = 0;
+    qs_time(&client_rec->start_s);
+    client_rec->end_s = client_rec->start_s + 1; // +1 prevents div by 0
+    client_rec->firstLine = m_lines;
     qsInitEvent(pool, client_rec->events);
     apr_table_setn(m_client_entries, tid, (char *)client_rec);
+  } else {
+    qs_time(&client_rec->end_s);
   }
+  client_rec->lastLine = m_lines;
   client_rec->request_count++;
   client_rec->duration += tme;
+  client_rec->duration_count_ms += tmems;
   if(k != NULL) {
     if(k[0] == '0' && k[1] == '\0') {
       client_rec->connections++;
@@ -806,6 +873,29 @@ static void updateClient(apr_pool_t *pool, char *T, char *t, char *D, char *S,
   } else {
     client_rec->duration_6++;
   }
+  if(B != NULL) {
+    client_rec->byte_count += atol(B);
+  }
+  if(ct) {
+    if(qsstrcasestr(ct, "html")) {
+      client_rec->html++;
+    } else if(qsstrcasestr(ct, "image")) {
+      client_rec->img++;
+    } else if(qsstrcasestr(ct, "css")) {
+      client_rec->cssjs++;
+    } else if(qsstrcasestr(ct, "javascript")) {
+      client_rec->cssjs++;
+    } else {
+      client_rec->other++;
+    }
+  }
+  if(m) {
+    if(strcasecmp(m, "get") == 0) {
+      client_rec->get++;
+    } else if(strcasecmp(m, "post") == 0) {
+      client_rec->post++;
+    }
+  }
   if(S != NULL) {
     if(strcmp(S, "200") != 0 && strcmp(S, "304") != 0 && strcmp(S, "302") != 0) {
       client_rec->error_count++;
@@ -818,6 +908,9 @@ static void updateClient(apr_pool_t *pool, char *T, char *t, char *D, char *S,
       client_rec->status_2++;
     } else if(S[0] == '3') {
       client_rec->status_3++;
+      if(S[1] == '0' && S[2] == '4') {
+        client_rec->status_304++;
+      }
     } else if(S[0] == '4') {
       client_rec->status_4++;
     } else if(S[0] == '5') {
@@ -964,11 +1057,16 @@ static void updateStat(apr_pool_t *pool, const char *cstr, char *line) {
   char *a = NULL; /* avarage 1 */
   char *A = NULL; /* average 2 */
   char *E = NULL; /* events */
+  char *ct = NULL; /* content type */
+  char *m = NULL; /* method */
   const char *c = cstr;
   char *l = line;
   long tme;
   long tmems;
   if(!line[0]) return;
+  if(m_offline || m_offline_count) {
+    m_lines++;
+  }
   while(c[0]) {
     /* process known types */
     if(c[0] == '.') {
@@ -1007,6 +1105,14 @@ static void updateStat(apr_pool_t *pool, const char *cstr, char *line) {
       if(l != NULL && l[0] != '\0') {
         C = cutNext(&l);
       }
+    } else if(c[0] == 'c') {
+      if(l != NULL && l[0] != '\0') {
+        ct = cutNext(&l);
+      }
+    } else if(c[0] == 'm') {
+      if(l != NULL && l[0] != '\0') {
+        m = cutNext(&l);
+      }
     } else if(c[0] == 'R') {
       if(l != NULL && l[0] != '\0') {
         R = cutNext(&l);
@@ -1104,12 +1210,11 @@ static void updateStat(apr_pool_t *pool, const char *cstr, char *line) {
       updateRec(rec, T, t, D, S, s, a, A, BI, B, R, I, U, Q, E, k, C, tme, tmems);
     }
   } else {
-    updateClient(pool, T, t, D, S, BI, B, R, I, U, Q, E, k, C, tme);
+    updateClient(pool, T, t, D, S, BI, B, R, I, U, Q, E, k, C, ct, tme, tmems, m);
   }
   qs_csUnLock();
 
   if(m_verbose && (m_offline || m_offline_count)) {
-    m_lines++;
     printf("[%ld] I=[%s] U=[%s] B=[%s] i=[%s] S=[%s] T=[%ld] Q=[%s]\n", m_lines,
            I == NULL ? "(null)" : I,
            U == NULL ? "(null)" : U,
@@ -1253,6 +1358,10 @@ static void readStdinOffline(apr_pool_t *pool, const char *cstr) {
   char buf[32];
   time_t unitTime = 0;
   int line_len;
+  FILE *outdev = stdout;
+  if(m_offline_count) {
+    outdev = stderr;
+  }
   while(fgets(line, sizeof(line), stdin) != NULL) {
     time_t l_time;
     line_len = strlen(line) - 1;
@@ -1274,14 +1383,14 @@ static void readStdinOffline(apr_pool_t *pool, const char *cstr) {
     } if(l_time < unitTime) {
       /* leap in time... */
       updateStat(pool, cstr, line);
-      fprintf(stdout, "X");
-      fflush(stdout);
+      fprintf(outdev, "X");
+      fflush(outdev);
       unitTime = 0;
     } else {
       if(l_time > unitTime) {
         if(!m_verbose) {
-          fprintf(stdout, ".");
-          fflush(stdout);
+          fprintf(outdev, ".");
+          fflush(outdev);
         }
       }
       while(l_time > unitTime) {
@@ -1366,7 +1475,7 @@ static void usage(const char *cmd, int man) {
   if(man) {
     printf(".SH SYNOPSIS\n");
   }
-  qs_man_print(man, "%s%s -f <format_string> -o <out_file> [-p [-v]] [-x [<num>]] [-u <name>] [-m] [-c <path>]\n", man ? "" : "Usage: ", cmd);
+  qs_man_print(man, "%s%s -f <format_string> -o <out_file> [-p[c] [-v]] [-x [<num>]] [-u <name>] [-m] [-c <path>]\n", man ? "" : "Usage: ", cmd);
   printf("\n");
   if(man) {
     printf(".SH DESCRIPTION\n");
@@ -1391,6 +1500,7 @@ static void usage(const char *cmd, int man) {
   qs_man_println(man, "  - free memory (m) (not available for all platforms)\n");
   qs_man_println(man, "  - number of client ip addresses seen withn the last %d seconds (ip)\n", ACTIVE_TIME);
   qs_man_println(man, "  - number of different users seen withn the last %d seconds (usr)\n", ACTIVE_TIME);
+  qs_man_println(man, "  - number of events identified by the 'E' format character\n");
   qs_man_println(man, "  - number of mod_qos events within the last minute (qV=create session,\n");
   qs_man_print(man, "    qS=session pass, qD=access denied, qK=connection closed, qT=dynamic\n");
   qs_man_print(man, "    keep-alive, qL=request/response slow down, qs=serialized request)\n");
@@ -1425,6 +1535,8 @@ static void usage(const char *cmd, int man) {
   qs_man_println(man, "     a arbitraty counter to build an average from (average per request)\n");
   qs_man_println(man, "     A arbitraty counter to build an average from (average per request)\n");
   qs_man_println(man, "     E comma separated list of event strings\n");
+  qs_man_println(man, "     c content type (%%{content-type}o), available in -pc mode only\n");
+  qs_man_println(man, "     m request method (GET/POST) (%%m), available in -pc mode only\n");
   qs_man_println(man, "     . defines an element to ignore (unknown string)\n");
   if(man) printf("\n.TP\n");
   qs_man_print(man, "  -o <out_file>\n");
@@ -1439,7 +1551,7 @@ static void usage(const char *cmd, int man) {
   qs_man_print(man, "     The option \"-pc\" may be used alternatively if you want to gather request\n");
   qs_man_print(man, "     information per client (identified by IP address (I) or user tracking id (U)\n");
   qs_man_print(man, "     showing how many request each client has performed within the captured period\n");
-  qs_man_print(man, "     of time).\n");
+  qs_man_print(man, "     of time). \"-pc\" supports the format characters IURSBTtDkEcm.\n");
   if(man) printf("\n.TP\n");
   qs_man_print(man, "  -v\n");
   if(man) printf("\n");
@@ -1452,7 +1564,7 @@ static void usage(const char *cmd, int man) {
   if(man) printf("\n.TP\n");
   qs_man_print(man, "  -u <name>\n");
   if(man) printf("\n");
-  qs_man_print(man, "     Become another user, e.g. www-data.\n");
+  qs_man_print(man, "     Becomes another user, e.g. www-data.\n");
   if(man) printf("\n.TP\n");
   qs_man_print(man, "  -m\n");
   if(man) printf("\n");
@@ -1586,10 +1698,17 @@ int main(int argc, const char *const argv[]) {
           // enable esco
           m_stat_rec->connections = 0;
         }
+        if(strchr(config, 'c')) {
+          // enable content type
+          m_ct = 1;
+        }
         if(strchr(config, 'D') || strchr(config, 't')) {
           // enable average duration in ms
           m_avms = 1;
         }
+        if(strchr(config, 'm')) {
+          m_methods = 1;
+        }
         if(strchr(config, 's') || strchr(config, 'a') || strchr(config, 'A')) {
           // enable custom counter
           m_customcounter = 1;
@@ -1662,11 +1781,13 @@ int main(int argc, const char *const argv[]) {
   if(m_offline_count) {
     int i;
     apr_table_entry_t *entry;
-    nice(10);
+    if(nice(10) == -1) {
+      fprintf(stderr, "ERROR, failed to change nice value: %s\n", strerror(errno));
+    }
     if(config == NULL) usage(cmd, 0);
     m_client_entries = apr_table_make(pool, MAX_CLIENT_ENTRIES);
     readStdinOffline(pool, config);
-    fprintf(stdout, ".\n");
+    fprintf(stderr, ".\n");
     entry = (apr_table_entry_t *) apr_table_elts(m_client_entries)->elts;
     m_f = stdout;
     if(file) {
@@ -1678,22 +1799,51 @@ int main(int argc, const char *const argv[]) {
     for(i = 0; i < apr_table_elts(m_client_entries)->nelts; i++) {
       client_rec_t *client_rec = (client_rec_t *)entry[i].val;
       char esco[256];
+      char m[256];
+      /* ci (coverage index): low value indicates that we have seen the client
+                              at the end or beginning of the file (maybe not all
+                              requests due to log rotation) */
+      long coverage = (client_rec->firstLine * 100 / m_lines);
+      long coverageend = 100 - ((client_rec->lastLine * 100) / m_lines);
+      if(coverageend < coverage) {
+        coverage = coverageend;
+      }
       esco[0] = '\0';
       if(m_stat_rec->connections != -1) {
         sprintf(esco, "esco;%ld;", client_rec->connections);
       }
-      fprintf(m_f, "%s;req;%ld;errors;%ld;"
-              "1xx;%ld;2xx;%ld;3xx;%ld;4xx;%ld;5xx;%ld;"
-              "av;%lld;<1s;%ld;1s;%ld;2s;%ld;3s;%ld;4s;%ld;5s;%ld;>5s;%ld;%s",
+      m[0] = '\0';
+      if(m_methods) {
+        sprintf(m, "GET;%ld;POST;%ld;",
+                client_rec->get,
+                client_rec->post);
+      }
+      if(m_avms == 0) {
+        // no ms available
+        client_rec->duration_count_ms = 1000 * client_rec->duration;
+      } else {
+        // improve accuracy (rounding errors):
+        client_rec->duration = client_rec->duration_count_ms / 1000;
+      }
+      fprintf(m_f, "%s;req;%ld;errors;%ld;duration;%ld;bytes;%lld;"
+              "1xx;%ld;2xx;%ld;3xx;%ld;4xx;%ld;5xx;%ld;304;%ld;"
+              "av;%lld;"NAVMS";%lld;<1s;%ld;1s;%ld;2s;%ld;3s;%ld;4s;%ld;5s;%ld;>5s;%ld;"
+              "%s"
+              "%s"
+              "ci;%ld;",
               entry[i].key,
               client_rec->request_count,
               client_rec->error_count,
+              client_rec->end_s - client_rec->start_s,
+              client_rec->byte_count,
               client_rec->status_1,
               client_rec->status_2,
               client_rec->status_3,
               client_rec->status_4,
               client_rec->status_5,
+              client_rec->status_304,
               client_rec->duration / client_rec->request_count,
+              client_rec->duration_count_ms / client_rec->request_count,
               client_rec->duration_0,
               client_rec->duration_1,
               client_rec->duration_2,
@@ -1701,7 +1851,16 @@ int main(int argc, const char *const argv[]) {
               client_rec->duration_4,
               client_rec->duration_5,
               client_rec->duration_6,
-              esco);
+              esco,
+              m,
+              coverage);
+      if(m_ct) {
+        fprintf(m_f, "html;%ld;css/js;%ld;img;%ld;other;%ld;",
+                client_rec->html,
+                client_rec->cssjs,
+                client_rec->img,
+                client_rec->other);
+      }
       if(apr_table_elts(client_rec->events)->nelts > 0) {
         int k;
         apr_table_entry_t *client_entry = (apr_table_entry_t *) apr_table_elts(client_rec->events)->elts;
@@ -1775,7 +1934,9 @@ int main(int argc, const char *const argv[]) {
    * the date string match of the log
    * enties. */
   if(m_offline) {
-    nice(10);
+    if(nice(10) == -1) {
+      fprintf(stderr, "ERROR, failed to change nice value: %s\n", strerror(errno));
+    }
     fprintf(stderr, "[%s]: offline mode (writes to %s)\n", cmd, file);
     m_date_str[0] = '\0';
     readStdinOffline(pool, config);
