@@ -27,7 +27,7 @@
  *
  */
 
-static const char revision[] = "$Id: qstail.c,v 1.12 2012/09/20 15:19:21 pbuchbinder Exp $";
+static const char revision[] = "$Id: qstail.c,v 1.14 2013/06/20 19:53:27 pbuchbinder Exp $";
 
 #include <stdio.h>
 #include <unistd.h>
@@ -50,7 +50,8 @@ static void usage(char *cmd, int man) {
   if(man) {
     printf(".SH NAME\n");
   }
-  qs_man_print(man, "Utility prints the end of a log file starting at the specified pattern.\n");
+  qs_man_print(man, "%s - an utility printing the end of a log file"
+               " starting at the specified pattern.\n", cmd);
   printf("\n");
   if(man) {
     printf(".SH SYNOPSIS\n");
@@ -123,7 +124,7 @@ static void qs_readline(long pos, FILE *f) {
 
 static int qs_tail(const char *cmd, FILE *f, const char *pattern) {
   char *cont = NULL;
-  long search_win_len = strlen(pattern) * 2 + 32;
+  long search_win_len = (strlen(pattern) * 2) + 32;
   char line[search_win_len + 10];
   long pos = 0;
   size_t len;
@@ -156,7 +157,7 @@ static int qs_tail(const char *cmd, FILE *f, const char *pattern) {
       s++;
       end = startpattern;
       /* search the end of the line */
-      while(end && end[0] && end[0] != CR && end[0] != LF) {
+      while((offset < search_win_len) && end[0] && end[0] != CR && end[0] != LF) {
         end++;
         offset++;
       }
